from math import acos,pi,asin
from d3g.distance import lineToLine,pointToLine,pointToPlane,p2p
from d3g.vector import magnitude,dotProduct,crossProduct
from d3g.angle import lines,planes,lineAndPlane
